BODY, TD {
  font-family: Verdana, Arial, Helvetica;
  font-size: 85%;
}

H1, H2, H3, H4, H5, H6 {
  display:inline;
}
PRE {
  background: #eee;
  border: 1px solid #888;
  padding: 4px;
}
CODE {
  background: #eee;
  /* border: 1px solid #888;
     padding: 1px; */
}


/* classes for syntax highlighting. */

/* copied from Adam Kennedy's PPI examples. */

.PPI span.word {
    color: darkslategray;
}
.PPI span.words {
    color: #999999;
}
.PPI span.transliterate {
    color: #9900FF;
}
.PPI span.substitute {
    color: #9900FF;
}
.PPI span.single {
    color: #999999;
}
.PPI span.regex {
    color: #9900FF;
}
.PPI span.pragma {
    color: #990000;
}
.PPI span.pod {
    color: #008080;
}
.PPI span.operator {
    color: #DD7700;
}
.PPI span.number {
    color: #990000;
}
.PPI span.match {
    color: #9900FF;
}
.PPI span.magic {
    color: #0099FF;
}
.PPI span.literal {
    color: #999999;
}
.PPI span.line_number {
    color: #666666;
}
.PPI span.keyword {
    color: #0000FF;
}
.PPI span.interpolate {
    color: #999999;
}
.PPI span.double {
    color: #999999;
}
.PPI span.core {
    color: #FF0000;
}
.PPI span.comment {
    color: #008080;
}
.PPI span.cast {
    color: #339999;
}


/* Copyright (c) 2005-2006 ActiveState Software Inc.
 *
 * Styles generated by ActiveState::Scineplex.
 *
 */

.SCINEPLEX span.comment {
  color:#ff0000;
  font-style: italic;
}

.SCINEPLEX span.default {
}
  
.SCINEPLEX span.keyword {
  color:#0099ff;
}
  
.SCINEPLEX span.here_document {
  color:#009933;
  font-weight: bold;	
}

.SCINEPLEX span.number {
  color:#8b0000;
  font-weight: bold;	
}
  
.SCINEPLEX span.operator {
  color:#0000ff;
  font-weight: bold;	
}
  
.SCINEPLEX span.regex {
  color:#c86400;
}
  
.SCINEPLEX span.string {
  color:#009933;
  font-weight: bold;	
}
  
.SCINEPLEX span.variable {
  color:0;
}
